package main

import (
	"crypto/sha512"
	"flag"
	"fmt"
	"encoding/binary"
	"time"
	"sync/atomic"
)

const Size512 = 64 // Size is the size, in bytes, of a SHA-512 checksum.

var (branch,     // число горутин
	  size,        // размер блока
	  width int)
var limit uint 

type mesage struct {
	solt uint32
	hash [Size512] byte
}	

func (m mesage) String() string { // формат вывода
	s := fmt.Sprintf("%07d | ", m.solt)
	for i, b:= range(m.hash) {
		if width > 0 && i > width {
			s += "..."
			break
		}
		s += fmt.Sprintf("%02X", b)
	}
	return s
}

func (m1 mesage) less(m2 mesage) bool {
	for i := 0; i < Size512; i++ {
		if m2.hash[i] < m1.hash[i] { return true  }
		if m2.hash[i] > m1.hash[i] { return false }
	}
	return true 
} 

func main() {
	exemplar := make(chan mesage, 30)   // канал результата
	var count uint32 = ^uint32(0)       // -1
	var finish uint32 = 0
	fmt.Printf("горутин %d; размер блока %d\n", branch, size)
	data := make([]byte, size)	
	for i, _:= range(data) { 		
		data[i] = byte(i % 256)
	}
	
	for _= range branch {
	   go func(ch chan mesage) {        // горутина хеш-функции
			var m mesage
	 		for {
				m.solt =	atomic.AddUint32(&count, 1)
				if m.solt > uint32(limit) { break }
            // заполняем 4 начальных байта блока:				
				binary.LittleEndian.PutUint32(data, m.solt)
				m.hash = sha512.Sum512(data)
				ch <- m
			}
			if atomic.AddUint32(&finish, 1) == uint32(branch) {
				close(ch)
			}
		}(exemplar)
	}

	tmStart := time.Now()
	dur := func() time.Duration {       // временная метка задержки
		return time.Now().Sub(tmStart)
	}
	first := true
	var m2 mesage
	for m1 := range exemplar {          // получить данные из канала
		if first {
			m2 = m1
			first = false
		}			
		if m2.less(m1) {
			fmt.Printf("% 14v : %s\n", dur(), m1)
			m2 = m1
		}
	}
	fmt.Printf("% 14v : завершение опроса\n", dur())
}

func init() {
	flag.IntVar(&branch, "g", 1, "число горутин")
	flag.IntVar(&size,   "b", 1000, "размер блока")
	flag.UintVar(&limit, "v", 32767, "лимит поиска")
	flag.IntVar(&width, "w", 0, "ширина вывода")
	flag.Parse()
}
