// https://ru.wikipedia.org/wiki/Алгоритм_Луна
// U.S. Patent 2 950 048 Computer for Verifying Numbers, Hans P. Luhn, August 23, 1960.

package main

import (
	"bufio"
	"errors"
	"fmt"
	"io"
	"os"
)

const LEN = 16

type card struct {
	num [LEN]byte
}

func (c card) valid() (val bool, err error) {
	val = false
	err = nil
	var s byte = 0
	for i, d := range c.num {
		if !(d >= byte('0') && d <= byte('9')) {
			s := fmt.Sprintf("ошибка: не число!")
			err = errors.New(s)
			return
		} else {
			c.num[i] = c.num[i] - byte('0')
			if i%2 == 0 {
				c.num[i] = c.num[i] * 2
				if c.num[i] > 9 {
					c.num[i] -= 9
				}
			}
			s += c.num[i]
		}
	}
	return s % 10 == 0, nil
}

func (c card) String() string { // формат вывода
	s := ""
	for i := range LEN {
		s += string(c.num[i])
	}
	return s
}

func main() {
	for {
		crd, err := incard()
		if err == nil {
			v, err := crd.valid()
			if err != nil {
				fmt.Println(err)
				continue
			}
			fmt.Printf("номер %s : ", crd)			
			if v {
				fmt.Printf("верифицирован\n")
			} else {
				fmt.Printf("не верифицирован\n")
			}

		} else if err == io.EOF {
			break
		} else {
			fmt.Println(err)
			break
		}
	}
}

func incard() (*card, error) {
	scanner := bufio.NewScanner(os.Stdin)
	if !scanner.Scan() {
		return nil, io.EOF // ввод ^D - EOF
	}
	line1 := scanner.Text()
	if 0 == len(line1) {
		return nil, io.EOF // ввод Enter - пустая строка
	}
	line2 := ""
	for _, c := range line1 {
		if c != ' ' {
			line2 += string(c)
		}
	}
	if len(line2) != LEN {
		err := errors.New(fmt.Sprintf("длина %d должна быть %d",
			len(line2), LEN))
		return nil, err
	}
	crd := new(card)
	for i, c := range line2 {
		crd.num[i] = byte(c)
	}
	return crd, nil
}
