package main

import (
	"container/ring"
	"fmt"
	"github.com/eiannone/keyboard"
	C "image/color"
	"unicode"
)

/*  type RGBA struct {  // структура цвета (импортируемая)
 *	   R, G, B, A uint8
 *  }
 */

type point struct {     // структура координат
	x, y float32
}

type pixel struct {
	point
	C.RGBA
}

func px(x, y float32, r, g, b, a uint8) pixel { // сокрашённая форма инциализации
	return pixel{point{x, y}, C.RGBA{r, g, b, a}}
}

func (p pixel) String() string { // формат вывода
	return fmt.Sprintf("[%5.2f,%5.2f]{%03v,%03v,%03v,%03v}",
		p.x, p.y, p.R, p.G, p.B, p.A)
}

type shape []pixel

func main() {
	s := shape{
      px(2, 0, 255, 0, 0, 255),
		px(0, 2, 127, 127, 0, 255),
		px(-2, 0, 0, 255, 0, 255),
		px(0, -2, 0, 127, 127, 255),
	}
	for i, x := range s {
		fmt.Printf("%d -> %v\n", i, x)
	}
	var r *ring.Ring              // голова циклического списка
	for i, z := range s {
		x := ring.New(1)
		x.Value = z
		if i == 0 {
			r = x
		} else {
			r = r.Link(x)
		}
	}
	show := func() {
		// Итерация поэлементно по всем элементам и их вывод
		r.Do(func(p any) { fmt.Print(p.(pixel), " ") })
		println()
	}
	show()

	keyboard.Open() // неблокирующий ввод с клавиатуры
	defer keyboard.Close()
	chChan := make(chan rune)     // канал ввода с клавиатуры
	go func(chChan chan<- rune) { // горутина ввода клавиши
		for {
			ch, _, _ := keyboard.GetKey()
			chChan <- ch
		}
	}(chChan)

	for {
		ch := <-chChan // вводимые символы
		if ch == 0 {
			break
		} else if unicode.IsControl(ch) {
			break
		} else if unicode.IsLetter(ch) {
			r = r.Next()
		} else if unicode.IsDigit(ch) {
			r = r.Prev()
		} else {
			break
		}
		show()
	}
	println()
}
